﻿using Syncfusion.Windows.Forms;
using Syncfusion.WinForms.DataGrid;
using Syncfusion.WinForms.DataGrid.Enums;
using Syncfusion.WinForms.DataGrid.Events;
using Syncfusion.WinForms.DataGrid.Interactivity;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SfDataGridDemo
{
    public partial class Form1 : Form
    {
        private readonly StringComparer _stringComparer = StringComparer.OrdinalIgnoreCase;
        private readonly SfToolTip _tooltip = new SfToolTip() { AutoPopDelay = 2000 };
        private IList<DatabaseTable> _tables;
        private DataTable _dataTable;
        private IList<Tuple<int, string>> _transformTypeValues;
        private SfDataGrid _columnsGrid;
        private SfDataGrid _transformsGrid;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            var integerFormatInfo = new NumberFormatInfo() { NumberDecimalDigits = 0, NumberGroupSizes = new int[] { } };

            tablesGrid.EditMode = EditMode.SingleClick;
            tablesGrid.AutoSizeColumnsMode = AutoSizeColumnsMode.Fill;
            tablesGrid.AutoGenerateColumns = false;
            tablesGrid.AllowDeleting = true;
            tablesGrid.AddNewRowPosition = RowPosition.Bottom;
            tablesGrid.Columns.Add(new GridComboBoxColumn() { MappingName = "Name", HeaderText = "Table Name", ValueMember = "TableName", DisplayMember = "TableName", Width = 300, ShowToolTip = true, ShowHeaderToolTip = true });
            tablesGrid.Columns.Add(new GridNumericColumn() { MappingName = "ObjectId", HeaderText = "Object Id", NumberFormatInfo = integerFormatInfo, AllowEditing = false });
            tablesGrid.Columns.Add(new GridNumericColumn() { MappingName = "Columns.Count", HeaderText = "Columns Count", NumberFormatInfo = integerFormatInfo, AllowEditing = false });
            tablesGrid.Columns.Add(new GridButtonColumn() { HeaderText = "Test Transform", DefaultButtonText = "Test Transform", AllowDefaultButtonText = true, MappingName = "Name" });
            tablesGrid.AddNewRowPosition = RowPosition.Bottom;
            tablesGrid.ToolTipOpening += Grid_ToolTipOpening;
            tablesGrid.RecordContextMenu = new ContextMenuStrip();
            tablesGrid.RecordContextMenu.Items.Add("Delete Row", null, tablesGrid_DeleteRowClicked);

            //columns
            _columnsGrid = new SfDataGrid
            {
                AutoGenerateColumns = false,
                AutoSizeColumnsMode = AutoSizeColumnsMode.Fill,
                EditMode = EditMode.SingleClick,
                AllowDeleting = false
            };

            _columnsGrid.Columns.Add(new GridTextColumn() { MappingName = "Name", HeaderText = "Column Name", AllowEditing = false, Width = 150, ShowHeaderToolTip = true, ShowToolTip = true });
            _columnsGrid.Columns.Add(new GridTextColumn() { MappingName = "DataType", HeaderText = "Data Type", AllowEditing = false });
            _columnsGrid.Columns.Add(new GridCheckBoxColumn() { MappingName = "IsSelected", HeaderText = "Is Selected", ShowHeaderToolTip = true });
            _columnsGrid.Columns.Add(new GridCheckBoxColumn() { MappingName = "IsPk", HeaderText = "Is Pk", AllowEditing = false });
            _columnsGrid.Columns.Add(new GridCheckBoxColumn() { MappingName = "IsIdentity", HeaderText = "Is Identity", AllowEditing = false });
            _columnsGrid.Columns.Add(new GridCheckBoxColumn() { MappingName = "IsComputed", HeaderText = "Is Computed", AllowEditing = false });
            _columnsGrid.Columns.Add(new GridNumericColumn() { MappingName = "Transforms.Count", HeaderText = "Transforms Count", NumberFormatInfo = integerFormatInfo, AllowEditing = false });
            _columnsGrid.ToolTipOpening += Grid_ToolTipOpening;
            _columnsGrid.RecordContextMenu = new ContextMenuStrip();
            _columnsGrid.RecordContextMenu.Items.Add("Delete Row", null, columnsGrid_DeleteRowClicked);

            tablesGrid.DetailsViewDefinitions.Add(new GridViewDefinition
            {
                RelationalColumn = "Columns",
                DataGrid = _columnsGrid
            });

            // tranforms
            _transformsGrid = new SfDataGrid
            {
                AutoGenerateColumns = false,
                AutoSizeColumnsMode = AutoSizeColumnsMode.Fill,
                AllowResizingColumns = true,
                EditMode = EditMode.SingleClick,
                AddNewRowPosition = RowPosition.Bottom,
                AllowDeleting = true
            };
            _transformsGrid.ToolTipOpening += Grid_ToolTipOpening;
            _transformsGrid.RecordContextMenu = new ContextMenuStrip();
            _transformsGrid.RecordContextMenu.Items.Add("Delete Row", null, transformsGrid_DeleteRowClicked);

            _transformsGrid.Columns.Add(new GridTextColumn() { MappingName = "Transform", HeaderText = "Transform", Width = 300, ShowHeaderToolTip = true });
            _transformsGrid.Columns.Add(new GridTextColumn() { MappingName = "WhereClause", HeaderText = "Where Clause", Width = 300, ShowHeaderToolTip = true });
            _transformsGrid.Columns.Add(new GridComboBoxColumn() { MappingName = "TransformType", HeaderText = "TransformType", ValueMember = "Item1", DisplayMember = "Item2", DataSource = _transformTypeValues, ShowHeaderToolTip = true });
            _transformsGrid.Columns.Add(new GridButtonColumn() { HeaderText = "Edit Transform", DefaultButtonText = "Edit Transform", AllowDefaultButtonText = true, MappingName = "Transform" });
            _columnsGrid.DetailsViewDefinitions.Add(new GridViewDefinition
            {
                RelationalColumn = "Transforms",
                DataGrid = _transformsGrid
            });

            SetupData();
            BindData();
        }

        private void Grid_ToolTipOpening(object sender, ToolTipOpeningEventArgs e)
        {
            e.ToolTipInfo.Items[0].Text = "";
            var tt = string.Empty;
            var grid = (SfDataGrid)sender;

            if (!_stringComparer.Equals(e.Column.HeaderText, "Column Name") || _stringComparer.Equals(e.DisplayText, "Column Name"))
            {
                // text from: https://loremipsum.io/generator/?n=5&t=p
                switch (e.Column.MappingName.ToLower())
                {
                    case "name":
                        tt = "The name of the object.";
                        break;
                    case "isselected":
                        tt = "Lorem ipsum dolor sit amet.";
                        break;
                    case "prescript":
                        tt = "Neque sodales ut etiam sit amet nisl purus in mollis.";
                        break;
                    case "postscript":
                        tt = "Ultrices in iaculis nunc sed augue.";
                        break;
                    case "scripttimeout":
                        tt = "Posuere urna nec tincidunt praesent semper feugiat nibh sed.";
                        break;
                    case "transform":
                        tt = "Cras adipiscing enim eu turpis.";
                        break;
                    case "whereclause":
                        tt = "Diam vel quam elementum pulvinar etiam non quam.";
                        break;
                    case "transformtype":
                        tt = "Blandit cursus risus at ultrices mi tempus imperdiet.";
                        break;
                    default:
                        break;
                }
            }
            else if (e.Record != null)
            {
                //Here get the child DataGrid
                var detailsViewDataGrid = e.OriginalSender as DetailsViewDataGrid;

                if (detailsViewDataGrid != null)
                {
                    //Here get the parentDataGrid
                    var parentDataGrid = detailsViewDataGrid.GetTopLevelParentDataGrid();

                    if (parentDataGrid != null)
                    {
                        //Here get the RowIndex of ChildDataGrid   
                        var childRowIndex = parentDataGrid.GetDetailsViewDataGridRowIndex(detailsViewDataGrid);

                        if (childRowIndex > -1)
                        {
                            //Here get the parent rowIndex by subtract 1 value from childRowIndex
                            var parentRowIndex = childRowIndex - 1;

                            if (parentRowIndex > -1)
                            {
                                //Here get the parent record index
                                var parentRecordIndex = parentDataGrid.TableControl.ResolveToRecordIndex(parentRowIndex);

                                if (parentRecordIndex > -1)
                                {
                                    //here get the parent record            
                                    var record = parentDataGrid.View?.Records[parentRecordIndex];

                                    if (record != null)
                                    {
                                        //Here know the properties of the primary item to which the secondary grid belongs.
                                        //Here you can access that main datagrid data
                                        var datatable = record.Data as DatabaseTable;
                                        var col = e.Record as DatabaseTableColumn;
                                        //Here you can assign the secondary grid the order id as maingrid order id                 
                                        tt = $"{datatable.Name} {col.Name} {col.DataType}";
                                    }
                                }
                            }
                        }
                    }
                }
            }
            e.ToolTipInfo.Items[0].Text = tt;
        }

        private void transformsGrid_DeleteRowClicked(object sender, EventArgs e)
        {
            Debug.WriteLine(nameof(transformsGrid_DeleteRowClicked));
            var grid = ((tablesGrid.SelectedDetailsViewGrid as SfDataGrid).SelectedDetailsViewGrid as SfDataGrid);
            var datasource = grid.DataSource as ObservableCollection<DatabaseTableColumnTransform>;
            datasource.Remove(((DatabaseTableColumnTransform)grid.CurrentItem));
            tablesGrid.CurrentItem = null;
            Debug.WriteLine($"SelectedIndex: {grid.SelectedIndex}, CurrentCell.ColumnIndex: {grid.CurrentCell.ColumnIndex}, CurrentCell.RowIndex: {grid.CurrentCell.RowIndex}");
        }

        private void columnsGrid_DeleteRowClicked(object sender, EventArgs e)
        {
            Debug.WriteLine(nameof(columnsGrid_DeleteRowClicked));
            var grid = (tablesGrid.SelectedDetailsViewGrid as SfDataGrid);
            Debug.WriteLine($"SelectedIndex: {grid.SelectedIndex}, CurrentCell.ColumnIndex: {grid.CurrentCell.ColumnIndex}, CurrentCell.RowIndex: {grid.CurrentCell.RowIndex}");
        }

        private void tablesGrid_DeleteRowClicked(object sender, EventArgs e)
        {
            Debug.WriteLine(nameof(tablesGrid_DeleteRowClicked));
            Debug.WriteLine($"SelectedIndex: {tablesGrid.SelectedIndex}, CurrentCell.RowIndex: {tablesGrid.CurrentCell.RowIndex}");
        }

        private void BindData()
        {
            var tablesCombo = (GridComboBoxColumn)tablesGrid.Columns["Name"];
            tablesCombo.MappingName = "Name";
            tablesCombo.ValueMember = "TableName";
            tablesCombo.DisplayMember = "TableName";
            tablesCombo.DataSource = _dataTable;

            tablesGrid.DataSource = _tables;
        }

        private void SetupData()
        {
            //this would normally get populated from a database
            _tables = new List<DatabaseTable>
            {
                new DatabaseTable(){
                    Name = "dbo.Person",
                    ObjectId = 1 ,
                    Columns = new ObservableCollection<DatabaseTableColumn>
                    {
                        new DatabaseTableColumn(){ Name = "PersonId", SqlDbType= SqlDbType.Int },
                        new DatabaseTableColumn(){
                            Name = "FirstName",
                            SqlDbType= SqlDbType.VarChar,
                            MaxLength = 100,
                            Transforms = new ObservableCollection<DatabaseTableColumnTransform>
                            {
                                new DatabaseTableColumnTransform() { Transform = "testing count display", TransformType = 1 },
                                new DatabaseTableColumnTransform() { Transform = "testing count display", TransformType = 2 }
                            }
                        },
                        new DatabaseTableColumn(){ Name = "LastName", SqlDbType= SqlDbType.VarChar, MaxLength = 100 }
                    }
                },
                new DatabaseTable()
                {
                    Name = "dbo.Company",
                    ObjectId = 2,
                    Columns = new ObservableCollection<DatabaseTableColumn>
                    {
                        new DatabaseTableColumn(){ Name = "CompanyId", SqlDbType= SqlDbType.Int },
                        new DatabaseTableColumn(){ Name = "Name", SqlDbType= SqlDbType.VarChar, MaxLength = 100 },
                        new DatabaseTableColumn()
                        {
                            Name = "Description",
                            SqlDbType= SqlDbType.VarChar,
                            MaxLength = -1,
                            Transforms = new ObservableCollection<DatabaseTableColumnTransform>
                            {
                                new DatabaseTableColumnTransform() { Transform = "testing count display 1", TransformType = 1 },
                                new DatabaseTableColumnTransform() { Transform = "testing count display 2", TransformType = 2 },
                                new DatabaseTableColumnTransform() { Transform = "testing count display 3", TransformType = 2 }
                            }
                        }
                    }
                }
            };

            _dataTable = new DataTable();
            _dataTable.Columns.Add("TableName", typeof(string));
            _dataTable.Rows.Add("dbo.Person");
            _dataTable.Rows.Add("dbo.Company");
            _dataTable.Rows.Add("dbo.Address");

            _transformTypeValues = new List<Tuple<int, string>>
            {
                Tuple.Create(0, "Type 1"),
                Tuple.Create(1, "Type 2"),
                Tuple.Create(2, "Type 3")
            };
        }

        private void button1_Click(object sender, EventArgs e)
        {
            SetupData();
            BindData();
        }
    }


    public class DatabaseTable
    {
        public string Name { get; set; }
        public int ObjectId { get; set; }
        public ObservableCollection<DatabaseTableColumn> Columns { get; set; } = new ObservableCollection<DatabaseTableColumn>();
    }

    public class DatabaseTableColumn
    {
        public string Name { get; set; }
        public SqlDbType SqlDbType { get; set; }
        public bool IsIdentity { get; set; }
        public bool IsPk { get; set; }
        public int PkOrdinal { get; set; }
        public int MaxLength { get; set; }
        public bool IsComputed { get; set; }
        public byte Scale { get; set; }
        public byte Precision { get; set; }
        public ObservableCollection<DatabaseTableColumnTransform> Transforms { get; set; } = new ObservableCollection<DatabaseTableColumnTransform>();
        public string DataType
        {
            get
            {
                switch (this.SqlDbType)
                {
                    case SqlDbType.Char:
                    case SqlDbType.Binary:
                    case SqlDbType.NChar:
                    case SqlDbType.NVarChar:
                    case SqlDbType.VarBinary:
                    case SqlDbType.VarChar:
                        return $"{this.SqlDbType} ({(this.MaxLength == -1 ? "MAX" : $"{this.MaxLength}")})";
                    case SqlDbType.Decimal:
                        return $"{this.SqlDbType} ({this.Precision}, {this.Scale})";
                    case SqlDbType.Time:
                    case SqlDbType.DateTime2:
                    case SqlDbType.DateTimeOffset:
                        return $"{this.SqlDbType} ({this.Scale})";
                    default:
                        return $"{this.SqlDbType}";
                }
            }
        }
    }

    public class DatabaseTableColumnTransform
    {
        public string Transform { get; set; }
        public int TransformType { get; set; }
    }
}
